@@ -31,6 +31,10 @@
 #include "arch.h"
 #include "db.h"
 
+/* state values */
+#define _DB_STA_VALID			0xA1B2C3D4
+#define _DB_STA_FREED			0x1A2B3C4D
+
 /* the priority field is fairly simple - without any user hints, or in the case
  * of a hint "tie", we give higher priority to syscalls with less chain nodes
  * (filter is easier to evaluate) */
@@ -306,6 +310,43 @@ int db_action_valid(uint32_t action)
 	return -EINVAL;
 }
 
+/**
+ * Free and reset the seccomp filter DB
+ * @param db the seccomp filter DB
+ * @param def_action the default filter action
+ *
+ * This function frees any existing filters and resets the filter DB to a
+ * default state; only the DB architecture is preserved.
+ *
+ */
+void db_reset(struct db_filter *db, uint32_t def_action)
+{
+	struct db_sys_list *s_iter;
+
+	if (db == NULL)
+		return;
+
+	/* free any filters */
+	if (db->syscalls != NULL) {
+		s_iter = db->syscalls;
+		while (s_iter != NULL) {
+			db->syscalls = s_iter->next;
+			_db_tree_free(s_iter->chains);
+			free(s_iter);
+			s_iter = db->syscalls;
+		}
+		db->syscalls = NULL;
+	}
+
+	/* set the default attribute values */
+	db->attr.act_default = def_action;
+	db->attr.act_badarch = SCMP_ACT_KILL;
+	db->attr.nnp_enable = 1;
+
+	/* set the state */
+	db->state = _DB_STA_VALID;
+}
+
 /**
  * Intitalize a seccomp filter DB
  * @param arch the architecture definition
@@ -322,13 +363,13 @@ struct db_filter *db_init(const struct arch_def *arch, uint32_t def_action)
 	db = malloc(sizeof(*db));
 	if (db == NULL)
 		return NULL;
+
+	/* clear the buffer for the first time and set the arch */
 	memset(db, 0, sizeof(*db));
 	db->arch = arch;
 
-	/* default attribute values */
-	db->attr.act_default = def_action;
-	db->attr.act_badarch = SCMP_ACT_KILL;
-	db->attr.nnp_enable = 1;
+	/* reset the DB to a known state */
+	db_reset(db, def_action);
 
 	return db;
 }
@@ -343,21 +384,32 @@ struct db_filter *db_init(const struct arch_def *arch, uint32_t def_action)
  */
 void db_release(struct db_filter *db)
 {
-	struct db_sys_list *s_iter;
-
 	if (db == NULL)
 		return;
 
-	s_iter = db->syscalls;
-	while (s_iter != NULL) {
-		db->syscalls = s_iter->next;
-		_db_tree_free(s_iter->chains);
-		free(s_iter);
-		s_iter = db->syscalls;
-	}
+	/* set the state, just in case */
+	db->state = _DB_STA_FREED;
+
+	/* free and reset the DB */
+	db_reset(db, 0);
 	free(db);
 }
 
+/**
+ * Validate a filter DB
+ * @param db the seccomp filter DB
+ *
+ * This function validates a seccomp filter DB.  Returns zero if the DB is
+ * valid, negative values on failure.
+ *
+ */
+int db_valid(struct db_filter *db)
+{
+	if (db != NULL && db->state == _DB_STA_VALID)
+		return 0;
+	return -EINVAL;
+}
+
 /**
  * Get a filter attribute
  * @param db the seccomp filter DB
@@ -368,7 +420,7 @@ void db_release(struct db_filter *db)
  * on success, negative values on failure.
  *
  */
-int db_attr_get(struct db_filter *db,
+int db_attr_get(const struct db_filter *db,
 		enum scmp_filter_attr attr, uint32_t *value)
 {
 	switch (attr) {
